/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import com.sun.istack.NotNull;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.branch.BranchService;
import ru.ulstu.extractor.gitrepository.GitRepositoryRepository;
import ru.ulstu.extractor.gitrepository.GitRepositoryService;
import ru.ulstu.extractor.model.Branch;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.model.Repository;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

@Service
public class IndexService {
    private final static Logger LOG = LoggerFactory.getLogger(IndexService.class);
    private final static int COMMITS_PAGE_SIZE = 10;
    private final GitRepositoryService gitRepositoryService;
    private final GitRepositoryRepository gitRepositoryRepository;
    private final BranchService branchService;

    public IndexService(GitRepositoryService gitRepositoryService,
                        GitRepositoryRepository gitRepositoryRepository,
                        BranchService branchService) {
        this.gitRepositoryService = gitRepositoryService;
        this.gitRepositoryRepository = gitRepositoryRepository;
        this.branchService = branchService;
    }

    @Transactional
    public void index(@NotNull String repositoryUrl, @NotNull String branchName) throws GitAPIException, IOException {
        Repository repository = gitRepositoryRepository.findByUrl(repositoryUrl);
        if (repository == null) {
            repository = gitRepositoryRepository.save(new Repository(repositoryUrl));
        }
        Branch branch = branchService.findByRepositoryAndName(repository, branchName);
        if (branch == null) {
            branch = new Branch(repository, branchName);
        }
        branchService.save(branch, Collections.emptyList());
        int commitsFrom = 0;
        int commitsTo = COMMITS_PAGE_SIZE;
        List<Commit> commits = gitRepositoryService.getCommits(repositoryUrl, branchName, commitsFrom, commitsTo, true);
        while (!commits.isEmpty()) {
            LOG.debug("{} commits loaded.", commits.size());
            branchService.addCommits(branch, commits);
            LOG.debug("{} commits successfully saved. {} {}", commits.size(), repositoryUrl, branchName);
            commitsFrom += COMMITS_PAGE_SIZE;
            commitsTo += COMMITS_PAGE_SIZE;
            commits = gitRepositoryService.getCommits(repositoryUrl, branchName, commitsFrom, commitsTo, false);
        }
        LOG.debug("Complete indexing {} branch", branchName);
    }
}
